import 'package:hgg_base/src/utils/x_log.dart';

extension ExMap on Map {
  /// 获取 value 转 string
  String s(String key, [String defaultValue = '']) {
    final value = this[key];
    if (value is String) return value;
    if (value == null) {
      XLog.d2('$key=null => String:$defaultValue');
      return defaultValue;
    }
    XLog.d2('${value.runtimeType} $key=$value => String:$value');
    return value.toString();
  }

  /// 获取 value 转 double
  double d(String key, [double defaultValue = 0]) {
    final value = this[key];
    if (value is double) return value;
    if (value is int) return value.toDouble();
    if (value == null) {
      XLog.d2('$key=null => double:$defaultValue');
      return defaultValue;
    }
    final other = double.tryParse(value.toString()) ?? defaultValue;
    XLog.d2('${value.runtimeType} $key=$value => double:$other');
    return other;
  }

  /// 获取 value 转 int
  int i(String key, [int defaultValue = 0]) {
    final value = this[key];
    if (value is int) return value;
    if (value is double) return value.toInt();
    if (value == null) {
      XLog.d2('$key=null => int:$defaultValue');
      return defaultValue;
    }
    final other = int.tryParse(value.toString()) ?? defaultValue;
    XLog.d2('${value.runtimeType} $key=$value => int:$other');
    return other;
  }

  /// 获取 value 转 bool
  bool b(String key, [bool defaultValue = false]) {
    final value = this[key];
    if (value is bool) return value;
    XLog.d2('${value.runtimeType} $key=$value => bool:$defaultValue');
    return defaultValue;
  }

  /// 获取 value 转 bool
  Map m(String key, [Map defaultValue = const {}]) {
    final value = this[key];
    if (value is Map) return value;
    XLog.d2('${value.runtimeType} $key=$value => map:$defaultValue');
    return defaultValue;
  }

  /// 获取 value 转 bool
  List l(String key, [List defaultValue = const []]) {
    final value = this[key];
    if (value is List) return value;
    XLog.d2('${value.runtimeType} $key=$value => list:$defaultValue');
    return defaultValue;
  }
}
