import 'package:dio/dio.dart';
import 'package:hgg_base/src/utils/x_log.dart';
import 'package:hgg_base/src/utils/x_util.dart';

import 'resp_data.dart';

export 'package:dio/dio.dart';

///API拦截器
class ApiInterceptor extends InterceptorsWrapper {
  final RespData Function(Map) mapToRespData;
  final Function(String) doShowErrorMsg;
  final bool showLog;

  ApiInterceptor(this.mapToRespData, this.doShowErrorMsg,
      {this.showLog = false});

  var isShowErrorMsg; //是否 toast 错误信息

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    var requestUrl = options.baseUrl + options.path;
    var formData = options.data;

    Map<String, dynamic> showInfo = {'Header信息': options.headers};
    if (options.queryParameters.isNotEmpty) {
      showInfo['query'] = options.queryParameters;
    }
    if (formData != null) {
      showInfo['data'] = formData;
    }
    if (showLog) XLog.i(showInfo, '###${options.method}请求### $requestUrl');

    return handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    isShowErrorMsg = true;
    var requestUrl = response.requestOptions.uri.toString();
    requestUrl = Uri.decodeComponent(requestUrl);

    var statusCode = response.statusCode;
    if (statusCode != 200) {
      /// 非200会在http的onError()中
    } else {
      if (!requestUrl.contains('hideLog')) {
        if (showLog) XLog.json(response.data, '请求url:$requestUrl');
      }

      if (response.data is Map) {
        Map data = response.data;
        RespData respData = mapToRespData(data);
        if (respData.success) {
          // 将 data 外的数据放到 date 内部
          if (respData.containKeys != null &&
              data.length > respData.containKeys!.length) {
            data.forEach((key, value) {
              if (!respData.containKeys!.contains(key)) {
                if (respData.data is Map) {
                  respData.data[key] = value;
                }
              }
            });
          }
          response.data = respData.data;
          return handler.resolve(response);
        } else {
          if (requestUrl.contains('hideMsg')) {
            isShowErrorMsg = false;
          }
          return handleFailed(respData, handler, response.requestOptions);
        }
      } else {
        XLog.e(response.data, '请求的结果不是jsonMap');
        return handler.reject(DioError(
          requestOptions: response.requestOptions,
          error: '请求的结果不是jsonMap',
        ));
      }
    }
  }

  /// 网络相关错误
  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    if (err.response == null) {
      if (!err.message.startsWith('CodeError:'))
        XLog.e(err.message, '###空Response错误### uri:${err.requestOptions.path}');
      return handler.reject(err);
    }
    int? statusCode = err.response?.statusCode;
    String? errorMsg = err.response?.statusMessage;
    switch (statusCode) {
      case 401: // 用非 200 状态吗,确定未登录
        RespData respData = mapToRespData(err.response?.data);
        respData.onUnLogin?.call();
        break;
      default:
        errorMsg = 'code:$statusCode\n$errorMsg';
        break;
    }
    XLog.e(err.response, '###非200错误###');
    return handler.reject(err);
  }

  /// 后台接口返回错误码
  void handleFailed(RespData respData, ResponseInterceptorHandler handler,
      RequestOptions requestOptions) {
    // 需要登录,  用接口内部 code ,确定未登录
    if (respData.code.toString() == respData.unLoginCode.toString()) {
      if (isShowErrorMsg) {
        doShowErrorMsg(respData.message);
      }
      respData.onUnLogin?.call();
    }

    // XLog.e(respData, '###后端code错误###');
    if (respData.onCodeError != null) {
      respData.onCodeError?.call(respData.code);
    } else {
      if (isShowErrorMsg && U.isNotEmpt(respData.message)) {
        //显示请求错误的信息
        doShowErrorMsg(respData.message);
      } else {
        XLog.e(respData.message, '未显示的code错误');
      }
    }
    return handler.reject(DioError(requestOptions: requestOptions,
        // error: 'CodeError-[${respData.code}]: ${respData.message}',
        error: {'code': respData.code, 'msg': respData.message}));
  }
}
