import 'dart:convert' as jsonk;

import 'package:logger/logger.dart';

/// Log Util.
class XLog {
  static Logger? _logger;
  static Logger? _logger2;
  static bool showMapLog = false;

  /// 初始化是否显示日志,debuggable true, log不输出.
  static void _init() {
    _logger = Logger(
        printer: PrefixPrinter(
            PrettyPrinter(
              methodCount: 1,
              errorMethodCount: 8,
              lineLength: 120,
              colors: false,
              printEmojis: true,
              printTime: false,
            ),
            verbose: 'JSON'),
        output: ConsoleOutput());
  }

  /// 初始化是否显示日志,debuggable true, log不输出.
  static void _init2() {
    _logger2 = Logger(
        printer: PrefixPrinter(
            PrettyPrinter(
              methodCount: 2,
              errorMethodCount: 8,
              lineLength: 120,
              colors: false,
              printEmojis: true,
              printTime: false,
            ),
            verbose: 'JSON'),
        output: ConsoleOutput());
  }

  /// 输出日志
  static void i(Object? object, [String? tag]) {
    if (_logger == null) {
      _init();
    }
    _logger?.i(object, tag);
  }

  /// 输出日志
  static void d(Object? object, [String? tag]) {
    if (_logger == null) {
      _init();
    }
    _logger?.d(object, tag);
  }

  static void d2(Object? object, [String? tag]) {
    if (showMapLog) {
      if (_logger2 == null) {
        _init2();
      }
      _logger2?.d(object, tag);
    }
  }

  /// 输出日志
  static void w(Object? object, [String? tag]) {
    if (_logger == null) {
      _init();
    }
    _logger?.w(object, tag);
  }

  static void long(Object? object, [String? tag]) {
    final logger = Logger(
      printer: PrefixPrinter(
          PrettyPrinter(
            methodCount: 1,
            errorMethodCount: 8,
            lineLength: 120,
            colors: false,
            printEmojis: true,
            printTime: false,
          ),
          verbose: 'JSON'),
    );
    logger.d(object, tag);
  }

  /// 输出调试信息
  static void e(Object? object, [String? tag]) {
    if (_logger == null) {
      _init();
    }
    _logger?.e(object, tag);
  }

  /// 输出日志
  static void json(Object? object, [String? tag]) {
    if (_logger == null) {
      _init();
    }
    _logger?.v(object, tag);
  }

  /// 打印显示json
  static void jsonShow(String jsonStr, String requestUri) {
    if (jsonStr.isEmpty) {
      _printLog('Empty/Null json content', 'log_util');
      return;
    }

    if (jsonStr.startsWith('{')) {
      Map<String, dynamic> jsonData = jsonk.json.decode(jsonStr);
      _printLog('{');
      _printMap(jsonData, 1);
      _printLog('}');
      _printLog('###DEBUG请求结束### $requestUri');
      return;
    }
    if (jsonStr.startsWith('[')) {
      List<Map<String, dynamic>> jsonData = jsonk.json.decode(jsonStr);
      _printLog('[');
      _printMapList(jsonData, 1);
      _printLog(']');
      _printLog('###DEBUG请求结束### $requestUri');
      return;
    }
    _printLog('不是Json', 'log_util');
  }

  static void _printLog(Object? object, [String? tag]) {
    String show = object.toString();
    String _tag = (tag == null || tag.isEmpty) ? '' : '[$tag]: ';

    var maxLength = 2000;
    while (show.isNotEmpty) {
      if (show.length > maxLength) {
        print('$_tag${show.substring(0, maxLength)}');
        show = show.substring(maxLength, show.length);
      } else {
        print('$_tag$show');
        show = '';
      }
    }
  }

  static void _printMapList(List jsonObjList, int k) {
    final sp = '    ';
    var biao = ',';
    int a = 0;
    jsonObjList.forEach((map) {
      if (a == jsonObjList.length - 1) {
        biao = '';
      }
      a++;
      if (map is Map) {
        _printLog('${sp * k}{');
        _printMap(map, k + 1);
        _printLog('${sp * k}}$biao');
      } else {
        _printLog('${sp * k}"$map"');
      }
    });
  }

  static void _printMap(Map jsonObj, int k) {
    final sp = '    ';
    var biao = ',';
    int a = 0;
    jsonObj.forEach((key, value) {
      if (a == jsonObj.length - 1) {
        biao = '';
      }
      a++;
      if (value is String) {
        String valueStr = value;
        if (valueStr.contains('"')) {
          valueStr = valueStr.replaceAll('"', '\\"');
        }
        if (valueStr.length > 512) {
          final sub = valueStr.substring(0, 30);
          valueStr = '$sub###很长的字符串###';
        }

        _printLog('${sp * k}"$key":"$valueStr"$biao');
      } else if (value.toString().startsWith('{')) {
        _printLog('${sp * k}"$key":{');
        _printMap(value, k + 1);
        _printLog('${sp * k}}$biao');
      } else if (value.toString().startsWith('[')) {
        _printLog('${sp * k}"$key":[');
        _printMapList(value, k + 1);
        _printLog('${sp * k}]$biao');
      } else {
        _printLog('${sp * k}"$key":$value$biao');
      }
    });
  }
}

/// 控制输出
class ConsoleOutput extends LogOutput {
  @override
  void output(OutputEvent event) {
    event.lines.forEach((str) {
      if (str.length > 900) {
        final sub = str.substring(0, 300);
        str = '$sub-------[### 很长的字符串 ###]';
      }
      print(str);
    });
  }
}

/// 美化显示
class PrettyPrinter extends LogPrinter {
  static const topLeftCorner = '┌';
  static const bottomLeftCorner = '└';
  static const middleCorner = '├';
  static const verticalLine = '│';
  static const doubleDivider = '─';
  static const singleDivider = '┄';

  static final levelColors = {
    Level.verbose: AnsiColor.fg(AnsiColor.grey(0.5)),
    Level.debug: AnsiColor.none(),
    Level.info: AnsiColor.fg(12),
    Level.warning: AnsiColor.fg(208),
    Level.error: AnsiColor.fg(196),
    Level.wtf: AnsiColor.fg(199),
  };

  static final levelEmojis = {
    Level.verbose: '',
    Level.debug: '🐛 ',
    Level.info: '', //'💡 '
    Level.warning: '⚠️ ',
    Level.error: '⛔ ',
    Level.wtf: '👾 ',
  };

  /// Matches a stacktrace line as generated on Android/iOS devices.
  /// For example:
  /// #1      Logger.log (package:logger/src/logger.dart:115:29)
  static final _deviceStackTraceRegex = RegExp(r'#[0-9]+[\s]+(.+) \(([^\s]+)\)');

  /// Matches a stacktrace line as generated by Flutter web.
  /// For example:
  /// packages/logger/src/printers/pretty_printer.dart 91:37
  static final _webStackTraceRegex = RegExp(r'^((packages|dart-sdk)\/[^\s]+\/)');

  /// Matches a stacktrace line as generated by browser Dart.
  /// For example:
  /// dart:sdk_internal
  /// package:logger/src/logger.dart
  static final _browserStackTraceRegex = RegExp(r'^(?:package:)?(dart:[^\s]+|[^\s]+)');

  static DateTime? _startTime;

  /// The index which to begin the stack trace at
  ///
  /// This can be useful if, for instance, Logger is wrapped in another class and
  /// you wish to remove these wrapped calls from stack trace
  final int stackTraceBeginIndex;
  final int methodCount;
  final int errorMethodCount;
  final int lineLength;
  final bool colors;
  final bool printEmojis;
  final bool printTime;

  String _topBorder = '';
  String _middleBorder = '';
  String _bottomBorder = '';

  PrettyPrinter({
    this.stackTraceBeginIndex = 0,
    this.methodCount = 2,
    this.errorMethodCount = 8,
    this.lineLength = 120,
    this.colors = true,
    this.printEmojis = true,
    this.printTime = false,
  }) {
    _startTime ??= DateTime.now();

    var doubleDividerLine = StringBuffer();
    var singleDividerLine = StringBuffer();
    for (var i = 0; i < lineLength - 1; i++) {
      doubleDividerLine.write(doubleDivider);
      singleDividerLine.write(singleDivider);
    }

    _topBorder = '$topLeftCorner$doubleDividerLine';
    _middleBorder = '$middleCorner$singleDividerLine';
    _bottomBorder = '$bottomLeftCorner$doubleDividerLine';
  }

  @override
  List<String> log(LogEvent event) {
    var messageStr = stringifyMessage(event.message);

    String? stackTraceStr;
    if (event.stackTrace == null) {
      if (methodCount > 0) {
        stackTraceStr = formatStackTrace(StackTrace.current, methodCount);
      }
    } else if (errorMethodCount > 0) {
      stackTraceStr = formatStackTrace(event.stackTrace, errorMethodCount);
    }

    var errorStr = event.error?.toString();

    String? timeStr;
    if (printTime) {
      timeStr = getTime();
    }

    return _formatAndPrint(
      event.level,
      messageStr,
      timeStr,
      errorStr,
      stackTraceStr,
    );
  }

  String? formatStackTrace(StackTrace? stackTrace, int methodCount) {
    var lines = stackTrace.toString().split('\n');
    if (stackTraceBeginIndex > 0 && stackTraceBeginIndex < lines.length - 1) {
      lines = lines.sublist(stackTraceBeginIndex);
    }
    var formatted = <String>[];
    var count = 0;
    for (var line in lines) {
      if (_discardDeviceStacktraceLine(line) ||
          _discardWebStacktraceLine(line) ||
          _discardBrowserStacktraceLine(line) ||
          line.isEmpty) {
        continue;
      }
      if (count > 1) // hgg 跳过显示工具类的位置
        formatted.add('${line.replaceFirst(RegExp(r'#\d+\s+'), '')}');
      if (++count == methodCount + 2) {
        break;
      }
    }

    if (formatted.isEmpty) {
      return null;
    } else {
      return formatted.join('\n');
    }
  }

  bool _discardDeviceStacktraceLine(String line) {
    var match = _deviceStackTraceRegex.matchAsPrefix(line);
    if (match == null) {
      return false;
    }
    return match.group(2)!.startsWith('package:logger');
  }

  bool _discardWebStacktraceLine(String line) {
    var match = _webStackTraceRegex.matchAsPrefix(line);
    if (match == null) {
      return false;
    }
    return match.group(1)!.startsWith('packages/logger') ||
        match.group(1)!.startsWith('dart-sdk/lib');
  }

  bool _discardBrowserStacktraceLine(String line) {
    var match = _browserStackTraceRegex.matchAsPrefix(line);
    if (match == null) {
      return false;
    }
    return match.group(1)!.startsWith('package:logger') || match.group(1)!.startsWith('dart:');
  }

  String getTime() {
    String _threeDigits(int n) {
      if (n >= 100) return '$n';
      if (n >= 10) return '0$n';
      return '00$n';
    }

    String _twoDigits(int n) {
      if (n >= 10) return '$n';
      return '0$n';
    }

    var now = DateTime.now();
    var h = _twoDigits(now.hour);
    var min = _twoDigits(now.minute);
    var sec = _twoDigits(now.second);
    var ms = _threeDigits(now.millisecond);
    var timeSinceStart = now.difference(_startTime!).toString();
    return '$h:$min:$sec.$ms (+$timeSinceStart)';
  }

  String stringifyMessage(dynamic message) {
    if (message is Map || message is Iterable) {
      var encoder = jsonk.JsonEncoder.withIndent('  ');
      try {
        return encoder.convert(message);
      } catch (e) {
        return message.toString();
      }
    } else {
      return message.toString();
    }
  }

  AnsiColor _getLevelColor(Level level) {
    if (colors) {
      return levelColors[level]!;
    } else {
      return AnsiColor.none();
    }
  }

  AnsiColor _getErrorColor(Level level) {
    if (colors) {
      if (level == Level.wtf) {
        return levelColors[Level.wtf]!.toBg();
      } else {
        return levelColors[Level.error]!.toBg();
      }
    } else {
      return AnsiColor.none();
    }
  }

  String _getEmoji(Level level) {
    if (printEmojis) {
      return levelEmojis[level]!;
    } else {
      return '';
    }
  }

  List<String> _formatAndPrint(
    Level level,
    String message,
    String? time,
    String? error,
    String? stacktrace,
  ) {
    // This code is non trivial and a type annotation here helps understanding.
    // ignore: omit_local_variable_types
    List<String> buffer = [];
    var color = _getLevelColor(level);
    buffer.add(color(_topBorder));

    if (error != null) {
      var errorColor = _getErrorColor(level);
      for (var line in error.split('\n')) {
        buffer.add(
          color('$verticalLine ') +
              errorColor.resetForeground +
              errorColor(line) +
              errorColor.resetBackground,
        );
      }
      buffer.add(color(_middleBorder));
    }

    if (stacktrace != null) {
      for (var line in stacktrace.split('\n')) {
        buffer.add('$color$verticalLine $line');
      }
      buffer.add(color(_middleBorder));
    }

    if (time != null) {
      buffer..add(color('$verticalLine $time'))..add(color(_middleBorder));
    }

    var emoji = _getEmoji(level);
    for (var line in message.split('\n')) {
      buffer.add(color('$verticalLine $emoji$line'));
    }
    buffer.add(color(_bottomBorder));

    return buffer;
  }
}
