import 'package:get_storage/get_storage.dart';

import 'x_log.dart';

class XSp {
  XSp._();

  /// 初始化
  static Future init() async {
    await GetStorage.init();
  }

  /// 保存数据
  static Future<void> put(String key, dynamic value) async {
    try {
      await GetStorage().write(key, value);
    } catch (e) {
      XLog.e('保存对象 ${value.runtimeType} 需要 toJson 函数');
    }
  }

  /// 获取数据
  static T? get<T>(String key, {T from(Map map)?, T? defValue}) {
    final read = GetStorage().read(key);
    if (read == null) {
      if (key != 'XSpKey.OssImageUrl') XLog.d('没有查询到数据,key=$key');
      return defValue;
    }
    if (read is Map) {
      if (read is T) {
        return read as T;
      }

      if (from == null) {
        XLog.d('需要将 jsonMap 转为 对象 $T 的 from 函数');
      }
      return from?.call(read) ?? defValue;
    }
    if (read is List) {
      XLog.e('需使用 getList 来获取List');
      return null;
    }
    return read;
  }

  /// 获取数据  //
  static List<T>? getList<T>(String key, [T from(Map map)?]) {
    final read = GetStorage().read(key);
    if (read == null) {
      XLog.d('没有查询到数据,key=$key');
      return null;
    }
    if (read is List) {
      if (read.isNotEmpty) {
        if (read.first is Map) {
          if (from != null) {
            return read.map((map) => from(map)).toList();
          } else {
            XLog.d('需要将 jsonMap 转为 对象 $T 的 from 函数');
            return null;
          }
        }
        return List.from(read);
      } else {
        return <T>[];
      }
    } else {
      XLog.e('存储的数据不是 List');
      return null;
    }
  }

  /// 是否已保存
  static bool hasData(String key) {
    return GetStorage().hasData(key);
  }

  /// 删除数据
  static Future<void> remove(String key) async {
    await GetStorage().remove(key);
  }

  /// 删除全部数据
  static Future<void> removeAll() async {
    await GetStorage().erase();
  }

  /// 显示存储的数据
  static show([String? showKey]) {
    if (showKey != null) {
      if (hasData(showKey)) {
        XLog.d({showKey: get(showKey)});
      } else {
        XLog.d('没有-$showKey-存储');
      }
      return;
    }

    final keys = GetStorage().getKeys<Iterable<String>>();
    final values = GetStorage().getValues();

    final map = {};
    for (var i = 0; i < keys.length; ++i) {
      final key = keys.elementAt(i);
      final value = values.elementAt(i);
      map[key] = value;
    }
    XLog.d(map);
  }
}
