import 'package:extended_image/extended_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:hgg_base/src/utils/x_size.dart';

/// 加载本地资源图片
class XImageAsset extends StatelessWidget {
  final String name;
  final double? width;
  final double? height;
  final BoxFit? fit;
  final Color? color;
  final BlendMode? colorBlendMode;
  final AlignmentGeometry alignment;
  final String? package;

  XImageAsset(
    this.name, {
    this.width,
    this.height,
    this.fit,
    this.color,
    this.colorBlendMode,
    this.alignment = Alignment.center,
    this.package,
  });

  @override
  Widget build(BuildContext context) {
    return Image.asset(
      name,
      width: D.w(width),
      height: D.h(height),
      fit: fit,
      color: color,
      colorBlendMode: colorBlendMode,
      alignment: alignment,
      package: package,
    );
  }

  /// 加载本地图片当背景
  static DecorationImage decorationImage(
    String name, {
    BoxFit? fit,
    AlignmentGeometry alignment = Alignment.center,
  }) {
    return DecorationImage(
      image: AssetImage(name),
      fit: fit,
      alignment: alignment,
    );
  }
}

/// 加载网络图片-base
class XImageNetBase extends StatelessWidget {
  final String imageUrl;
  final double? width;
  final double? height;
  final BoxFit? fit;
  final Widget? errorWidget;
  final Widget? placeholder;
  final String imagePrefix;
  final BorderRadius? borderRadius;

  XImageNetBase(
    this.imageUrl, {
    this.width,
    this.height,
    this.fit = BoxFit.cover,
    this.errorWidget,
    this.placeholder,
    this.imagePrefix = '',
    this.borderRadius,
  });

  @override
  Widget build(BuildContext context) {
    var image = imageUrl;

    if (image.isEmpty) {
      if (errorWidget != null) {
        return errorWidget!;
      }
    }
    if (!image.startsWith('http')) {
      image = imagePrefix + imageUrl.trim();
    }

    return ExtendedImage.network(
      image,
      width: D.w(width),
      height: D.h(height),
      fit: fit,
      borderRadius: borderRadius,
      shape: borderRadius != null ? BoxShape.rectangle : null,
      cache: true,
      // compressImage: _compressImage,
      loadStateChanged: (ExtendedImageState state) {
        switch (state.extendedImageLoadState) {
          case LoadState.loading:
            return placeholder ??
                Container(
                  color: Colors.grey[200],
                  width: D.w(width),
                  height: D.h(height),
                );
          case LoadState.completed:
            return state.completedWidget;
          case LoadState.failed:
            return errorWidget ??
                Container(
                  color: Colors.grey[200],
                  width: D.w(width),
                  height: D.h(height),
                );
          default:
            return null;
        }
      },
    );
  }

  /// 大图压缩后进行缓存
/*Future<Uint8List> _compressImage(Uint8List list) async {
    var result = await FlutterImageCompress.compressWithList(
      list,
      minHeight: 1920,
      minWidth: 1080,
      quality: 30,
    );
    return Uint8List.fromList(result);
  }*/
}
