import 'package:flutter_test/flutter_test.dart';
import 'package:hgg_base/src/utils/x_log.dart';
import 'package:hgg_base/src/utils/x_sp.dart';

void main() async {
  testXSp();
}

testXSp() {
  group('测试 XSP 相关', () {
    setUp(() async {
      // 伪造目录
      // XPathProviderPlatform.init();
      // 初始化库
      await XSp.init();
    });
    test('存入数据-int', () => XSp.put('The_Key_Int', 123));
    test('获取数据-int', () {
      final data = XSp.get<int>('The_Key_Int');
      XLog.d(data, data.runtimeType.toString());
    });

    test('存入数据-string', () => XSp.put('The_Key_String', '张三'));
    test('获取数据-String', () {
      final data = XSp.get<String>('The_Key_String');
      XLog.d(data, data.runtimeType.toString());
    });

    test('存入数据-Map',
        () => XSp.put('The_Key_Map', {'name': '张三', 'sex': '男', 'age': 22}));
    test('获取数据-Map', () {
      final data = XSp.get<Map<String, dynamic>>('The_Key_Map');
      XLog.d(data, data.runtimeType.toString());
    });

    test('存入数据-List', () => XSp.put('The_Key_List', [1, 2, 3]));
    test('获取数据-List', () {
      final data = XSp.getList<int>('The_Key_List');
      XLog.d(data, data.runtimeType.toString());
    });

    test('存入数据-Obj', () => XSp.put('The_Key_Obj', Student('小红', '女', 21)));
    test('获取数据-Obj', () {
      final data = XSp.get('The_Key_Obj', from: Student.fromJson);
      XLog.d(data, data.runtimeType.toString());
    });

    test(
        '存入数据-ObjList',
        () => XSp.put('The_Key_ObjList',
            [Student('小红', '女', 21), Student('小张', '男', 32)]));
    test('获取数据-ObjList', () {
      final data = XSp.getList('The_Key_ObjList', Student.fromJson);
      XLog.d(data, data.runtimeType.toString());
    });

    test('清空数据', () => XSp.removeAll(), skip: '单独调用');
  });
}

/// 测试用 [学生] 类
class Student {
  String name;
  String sex;
  int age;

  Student(this.name, this.sex, this.age);

  static Student fromJson(Map map) {
    return Student(map['name'], map['sex'], map['age']);
  }

  Map toJson() {
    return {'name': name, 'sex': sex, 'age': age};
  }

  @override
  String toString() {
    return 'Student{${toJson()}';
  }
}
